import os
from empatican.physio import e4
# from django.test import TestCase
from dateutil import parser
from datetime import datetime
import pandas as pd
import pytz
import tempfile
from unittest import TestCase

FIXTURES_DIR = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), 'fixtures')


class E4StreamTest(TestCase):
    # def setUp(self):
    #     print(self._testMethodName)

    def test_e4_stream_unzipped_csv(self):
        acc_csv = os.path.join(FIXTURES_DIR, 'e4', 'ACC.csv')
        example = dict(
            timestamp=1533751118,
            freq=32,
            n_samples=2000,
            last_datetime=parser.parse('2018-08-08 17:59:40.468750+0:00'))

        acc_stream = e4.AccStream(acc_csv)
        last_datetime = acc_stream.data.index[-1].to_pydatetime()

        self.assertEqual(acc_stream.start_timestamp, example['timestamp'])
        self.assertEqual(acc_stream.frequency, example['freq'])
        self.assertEqual(acc_stream.data.shape[0], example['n_samples'])
        self.assertEqual(list(acc_stream.data.columns), ['x', 'y', 'z'])
        self.assertEqual(last_datetime, example['last_datetime'])

    def test_e4_stream_zipped_csv(self):
        acc_csv = os.path.join(FIXTURES_DIR, 'e4', 'ACC.csv.gz')
        timestamp = 1533751118
        example = dict(
            timestamp=timestamp,
            freq=32,
            n_samples=2000,
            first_datetime=datetime.fromtimestamp(timestamp, tz=pytz.utc),
            last_datetime=parser.parse('2018-08-08 17:59:40.468750+0:00'))

        acc_stream = e4.AccStream(acc_csv)
        first_datetime = acc_stream.data.index[0].to_pydatetime()
        last_datetime = acc_stream.data.index[-1].to_pydatetime()

        self.assertEqual(acc_stream.start_timestamp, example['timestamp'])
        self.assertEqual(acc_stream.frequency, example['freq'])
        self.assertEqual(acc_stream.data.shape[0], example['n_samples'])
        self.assertEqual(list(acc_stream.data.columns), ['x', 'y', 'z'])
        self.assertEqual(first_datetime, example['first_datetime'])
        self.assertEqual(last_datetime, example['last_datetime'])

    def test_e4_bvp_csv(self):
        bvp_csv = os.path.join(FIXTURES_DIR, 'e4', 'BVP.csv')
        example_timestamp, example_freq, example_n_samples = (1533751118, 64,
                                                              2000)

        bvp_stream = e4.BvpStream(bvp_csv)
        self.assertEqual(bvp_stream.start_timestamp, example_timestamp)
        self.assertEqual(bvp_stream.frequency, example_freq)
        self.assertEqual(bvp_stream.data.shape[0], example_n_samples)
        self.assertEqual(bvp_stream.data.columns, ['bvp'])

    def test_e4_eda_csv(self):
        eda_csv = os.path.join(FIXTURES_DIR, 'e4', ('EDA.csv'))
        example_timestamp, example_freq, example_n_samples = (1533751118, 4,
                                                              200)

        eda_stream = e4.EdaStream(eda_csv)
        self.assertEqual(eda_stream.start_timestamp, example_timestamp)
        self.assertEqual(eda_stream.frequency, example_freq)
        self.assertEqual(eda_stream.data.shape[0], example_n_samples)
        self.assertEqual(eda_stream.data.columns, ['eda'])

    def test_e4_hr_csv(self):
        hr_csv = os.path.join(FIXTURES_DIR, 'e4', ('HR.csv'))
        example_timestamp, example_freq, example_n_samples = (1533751128, 1,
                                                              1579)

        hr_stream = e4.HrStream(hr_csv)
        self.assertEqual(hr_stream.start_timestamp, example_timestamp)
        self.assertEqual(hr_stream.frequency, example_freq)
        self.assertEqual(hr_stream.data.shape[0], example_n_samples)
        self.assertEqual(hr_stream.data.columns, ['hr'])

    def test_e4_ibi_csv(self):
        ibi_csv = os.path.join(FIXTURES_DIR, 'e4', ('IBI.csv'))

        timestamp = 1533751118
        first_example = datetime.fromtimestamp(
            timestamp, tz=pytz.utc).replace(microsecond=0)

        example = dict(
            timestamp=timestamp,
            freq=None,
            n_samples=121,
            first_datetime=first_example,
            last_datetime=parser.parse('2018-08-09T13:04:19.302850962+0:00'))

        ibi_stream = e4.IbiStream(ibi_csv)

        first_datetime = ibi_stream.data.index[0].replace(
            microsecond=0, nanosecond=0).to_pydatetime()
        last_datetime = ibi_stream.data.index[-1].replace(
            nanosecond=0).to_pydatetime()

        self.assertEqual(ibi_stream.start_timestamp, example['timestamp'])
        self.assertEqual(ibi_stream.frequency, example['freq'])
        self.assertEqual(ibi_stream.data.shape[0], example['n_samples'])
        self.assertEqual(ibi_stream.data.columns, ['ibi'])

        # TODO nb Testing of the first timestamp for a time-delta index is
        # really the initial timestamp plus the first offset
        self.assertEqual(first_datetime, example['first_datetime'])
        self.assertEqual(last_datetime,
                         example['last_datetime'].astimezone(pytz.utc))

    def test_e4_press_csv(self):
        csv = os.path.join(FIXTURES_DIR, 'e4', ('tags.csv'))

        timestamp = None
        example = dict(
            timestamp=timestamp,
            freq=None,
            n_samples=3,
            first_datetime=datetime.fromtimestamp(1533751266.23, tz=pytz.utc),
            last_datetime=datetime.fromtimestamp(1533752705.76, tz=pytz.utc))

        stream = e4.PressStream(csv)

        first_datetime = stream.data.index[0].replace(
            nanosecond=0).to_pydatetime()
        last_datetime = stream.data.index[-1].replace(
            nanosecond=0).to_pydatetime()

        self.assertEqual(stream.start_timestamp, example['timestamp'])
        self.assertEqual(stream.frequency, example['freq'])
        self.assertEqual(stream.data.shape[0], example['n_samples'])
        self.assertEqual(stream.data.columns, ['press'])
        self.assertEqual(first_datetime, example['first_datetime'])

        # Rounding error in nanoseconds with the last datetime

        self.assertEqual(last_datetime.year, example['last_datetime'].year)
        self.assertEqual(last_datetime.month, example['last_datetime'].month)
        self.assertEqual(last_datetime.day, example['last_datetime'].day)
        self.assertEqual(last_datetime.hour, example['last_datetime'].hour)
        self.assertEqual(last_datetime.minute, example['last_datetime'].minute)
        self.assertEqual(last_datetime.second, example['last_datetime'].second)

    def test_e4_temperature_csv(self):
        temperature_csv = os.path.join(FIXTURES_DIR, 'e4', ('TEMP.csv'))
        example_timestamp, example_freq, example_n_samples = (1533751118, 4,
                                                              6352)

        temperature_stream = e4.TemperatureStream(temperature_csv)
        self.assertEqual(temperature_stream.start_timestamp, example_timestamp)
        self.assertEqual(temperature_stream.frequency, example_freq)
        self.assertEqual(temperature_stream.data.shape[0], example_n_samples)
        self.assertEqual(temperature_stream.data.columns, ['temperature'])


class E4SessionTest(TestCase):
    def test_e4_session_dir(self):
        session_path = os.path.join(FIXTURES_DIR, 'e4')
        example = dict(timestamp=1533751118, hr_timestamp=1533751128)

        session = e4.Session(session_path)

        # Just check stream timestamps (more detailed testing of streams above)
        self.assertEqual(session.streams['acc'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['bvp'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['eda'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['hr'].start_timestamp,
                         example['hr_timestamp'])
        self.assertEqual(session.streams['ibi'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['temperature'].start_timestamp,
                         example['timestamp'])
        # nb Press / tags stream is odd and has no timestamp
        self.assertEqual(session.streams['press'].start_timestamp, None)
        self.assertEqual(session.streams['press'].num_samples, 3)

        # Check that session metadata was loaded appropriately
        self.assertEqual(session.e4_meta['device_id'], '54a32b')
        self.assertEqual(session.e4_meta['start_time'], 1533751118)
        self.assertEqual(session.e4_meta['duration'], 62.46875)
        self.assertEqual(session.e4_meta['label'], '5693')
        self.assertEqual(session.e4_meta['device'], 'E4 2.1')

        # Cleanup
        del (session)

    def test_e4_session_dir_nometa(self):
        session_path = os.path.join(FIXTURES_DIR, 'e4_nometa')
        example = dict(timestamp=1533751118, hr_timestamp=1533751128)

        session = e4.Session(session_path)

        # Just check stream timestamps (more detailed testing of streams above)
        self.assertEqual(session.streams['acc'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['bvp'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['eda'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['hr'].start_timestamp,
                         example['hr_timestamp'])
        self.assertEqual(session.streams['ibi'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['temperature'].start_timestamp,
                         example['timestamp'])
        # nb Press / tags stream is odd and has no timestamp
        self.assertEqual(session.streams['press'].start_timestamp, None)
        self.assertEqual(session.streams['press'].num_samples, 3)

        # E4 Meta should be empty
        self.assertEqual(session.e4_meta, {})

        # Cleanup
        del (session)

    def test_e4_session_zip(self):
        session_path = os.path.join(FIXTURES_DIR, 'zips', 'Sess01.zip')
        example = dict(timestamp=1533751118, hr_timestamp=1533751128)

        session = e4.Session(session_path)

        # Just check stream timestamps (more detailed testing of streams above)
        self.assertEqual(session.streams['acc'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['bvp'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['eda'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['hr'].start_timestamp,
                         example['hr_timestamp'])
        self.assertEqual(session.streams['ibi'].start_timestamp,
                         example['timestamp'])
        self.assertEqual(session.streams['temperature'].start_timestamp,
                         example['timestamp'])
        # nb Press / tags stream is odd and has no timestamp
        self.assertEqual(session.streams['press'].start_timestamp, None)
        self.assertEqual(session.streams['press'].num_samples, 3)

        del (session.source_path)

    def test_e4_session_ioerror(self):
        bad_path = os.path.join(FIXTURES_DIR, 'zips', 'Badpath.zip')

        with self.assertRaises(IOError):
            e4.Session(bad_path)

    def test_e4_session_to_hdf(self):
        session_path = os.path.join(FIXTURES_DIR, 'e4')
        session = e4.Session(session_path)
        h5_fname = 'session.h5'

        with tempfile.NamedTemporaryFile() as h5_tmp:
            h5_fname = h5_tmp.name
            session.to_hdf(h5_fname)

            self.assertTrue(os.path.getsize(h5_fname) > 0)

            # Read the new h5 file and make sure it was stored correctly
            with pd.HDFStore(h5_fname) as store:
                acc_attrs = store.get_storer('dt/acc').attrs
                self.assertEqual(session.streams['acc'].start_timestamp,
                                 acc_attrs.metadata['start_timestamp'])

    def test_e4_session_from_hdf(self):
        session_path = os.path.join(FIXTURES_DIR, 'e4')
        example_session = e4.Session(session_path)

        h5_fname = os.path.join(FIXTURES_DIR, 'zips/Sess01.h5')

        session = e4.Session(h5_fname)

        self.assertEqual(session.streams['acc'].start_timestamp,
                         example_session.streams['acc'].start_timestamp)
        self.assertEqual(session.streams['acc'].frequency,
                         example_session.streams['acc'].frequency)
        self.assertEqual(session.streams['acc'].num_samples,
                         example_session.streams['acc'].num_samples)
        self.assertEqual(
            os.path.basename(session.streams['acc'].filepath),
            os.path.basename(example_session.streams['acc'].filepath))

    def test_e4_stream_to_csv(self):
        acc_csv = os.path.join(FIXTURES_DIR, 'e4', 'ACC.csv')
        acc_stream = e4.AccStream(acc_csv)

        with tempfile.NamedTemporaryFile() as acc_tmp:
            # out_csv = 'ACC.csv'  # for quick checking
            out_csv = acc_tmp.name
            acc_stream.to_csv(out_csv)
            self.assertTrue(os.path.getsize(out_csv) > 0)

    def test_e4_session_to_csv(self):
        session_path = os.path.join(FIXTURES_DIR, 'e4')
        session = e4.Session(session_path)

        # Output as a full directory
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = os.path.join(tmp_dir, 'Session')
            # tmp_path = './Session'  # Local testing
            session.to_archive(tmp_path)
            self.assertTrue(os.path.exists(os.path.join(tmp_path, 'ACC.csv')))

        # Output as a zip archive
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = os.path.join(tmp_dir, 'Session.zip')
            # tmp_path = './Session.zip'  # Local testing
            session.to_archive(tmp_path)
            self.assertTrue(os.path.getsize(tmp_path) > 0)

        # Test IOError
        with tempfile.TemporaryDirectory() as tmp_dir:
            existing_dir = os.path.join(tmp_dir, 'ImHereAlready')
            os.makedirs(existing_dir)
            with self.assertRaises(IOError):
                session.to_archive(existing_dir)



#!/usr/bin/env python
import argparse
import dateutil.parser
from empatican.interface import connect
import sys


def add_parser():
    help = 'Search and Download E4 Sessions from Empatica Connect'
    parser = argparse.ArgumentParser(description=help)
    add_arguments(parser)
    return parser


def add_arguments(parser):
    subparsers = parser.add_subparsers(dest='action')
    list_parser = subparsers.add_parser('list')
    download_parser = subparsers.add_parser('download')
    add_query_args(list_parser)
    add_query_args(download_parser)

    list_parser.add_argument(
        '--table-fmt',
        default='tab',
        choices=['tab', 'csv'],
        help=('Display table as tab-separated (nice on screen) '
              'or csv (useful for saving output)'))

    download_parser.add_argument(
        '--template', '-t', help='Output template to use when saving zipfiles')


def add_query_args(parser):
    # parser.add_argument('action', choices=['download', 'list'])
    parser.add_argument(
        '--device-assignment', help='Participant->Device sheet as csv')

    description = 'Arguments used to filter or query specific sessions'
    group = parser.add_argument_group(title='query arguments', description=description)
    group.add_argument(
        '--device-id', '-d', nargs='*', help='Restrict to Device ID', type=str)
    group.add_argument(
        '--empatica-id',
        nargs='*',
        help='Space-separated list of Empatica Connect website session IDs')
    group.add_argument(
        '--participant-id',
        nargs='+',
        help='Space-separated list of Participant IDs to restict',
    )
    group.add_argument(
        '--n-recent', metavar='N', type=int, help='Number of recent sessions')
    group.add_argument(
        '--start-date',
        type=dateutil.parser.parse,
        metavar='YYYY-MM-DD',
        help='Date of earliest session')
    group.add_argument(
        '--end-date',
        type=dateutil.parser.parse,
        metavar='YYYY-MM-DD',
        help='Date of latest session')

    # parser.add_argument(
    #     '--userid', type=str, help='Optional Empatica user id (for performance)')


def main():
    parser = add_parser()
    args = parser.parse_args(sys.argv[1:])
    connect.search_or_download(vars(args))



"""
Classes for Empatica E4-formatted Physio import
"""
from __future__ import division  # In case of Py2k fix float division
# from Atheneum import models
import datetime
# from django.db import connection
import gzip
import json
import logging
import os
import pandas as pd
import pytz
from shutil import rmtree
import tempfile
import zipfile

logger = logging.getLogger(__name__)


class Session(object):
    """A full E4 physio session archive / zip containing several stream csv's.

    Arguments
    -----------
    source_path : file handle or path string
        Source of the physio session; a path to a directory or archive zip file
        containing E4 csv streams.

    Attributes
    ----------
    streams : dict
        A dictionary with stream names as keys and
        :class:`~empatican.physio.e4.Stream` objects as values

    start_datetime : datetime
        A "common" start time for most streams in the session.

    duration : float
        Length of stream recording in seconds


    Examples
    ---------
    >>> session1 = Session('Sub01')
    >>> session1.streams['acc']
    < physio.AccStream at ... >


    >>> session2 = Session('Sub01.zip')
    >>> session2.streams['eda']
    < physio.EdaStream at ... >

    Raises
    ------
    IOError : If the filename doesn't exist
    """

    def __init__(self, source_path):
        # if isinstance(source_path, models.Physio_Session):
        #     self.load_from_db(source_path)
        if 'endswith' in dir(source_path) and source_path.endswith('.h5'):
            self.load_hdf(source_path)
        else:
            self.load_csvs(source_path)

        start_datetime = datetime.datetime.fromtimestamp(
            self.streams['acc'].start_timestamp, tz=pytz.utc)
        self.start_datetime = start_datetime

        duration = self.streams['acc'].duration
        self.duration = duration

        assert ('e4_meta' in dir(self))

    def load_csvs(self, source_path, on_error="WARN"):
        self.source_path = source_path
        self.streams = self.read_streams()

        metadata_path = os.path.join(self._temp_source_path, 'metadata.json')
        e4_meta = self._load_meta(metadata_path)
        self.e4_meta = e4_meta

    @property
    def source_path(self):
        return self._source_path

    @source_path.setter
    def source_path(self, source_path):
        try:
            dirlike = os.path.isdir(source_path)
        except TypeError:
            dirlike = False

        if dirlike or self.hdf5like(source_path):
            self._temp_source_path = source_path
        elif zipfile.is_zipfile(source_path):
            with zipfile.ZipFile(source_path) as zf:
                tmpdir = tempfile.mkdtemp()
                zf.extractall(tmpdir)
            self._temp_source_path = tmpdir
        else:
            self._source_path, self._temp_source_path = None, None
            source_msg = ("Check path: File %s must be an existing directory, "
                          "hdf5 or zip file." % source_path)
            raise IOError(source_msg)
        self._source_path = source_path

    @source_path.deleter
    def source_path(self):
        if '_temp_source_path' in dir(self):
            if self._temp_source_path != self._source_path and os.path.isdir(
                    self._temp_source_path):
                logging.debug("Removing temperary extracted archive dir %s" %
                              self._temp_source_path)
                rmtree(self._temp_source_path)

    def read_streams(self):
        """Read all streams in an archive and expose them in a dict

        Returns
        -------

        streams : dict
            Dictionary of streams with stream names as keys and physio.Stream
            objects as values.
        """
        streams = dict()

        # Loop through each stream and instantiate it from the appropriate csv
        for stream_name, (stream_class, stream_path) in STREAM_SOURCE.items():
            temp_path = os.path.join(self._temp_source_path, stream_path)
            streams[stream_name] = stream_class(temp_path)

        return streams

    def __del__(self):
        # Ensure that temporary extracted files are removed
        # (does NOT delete the zip on disk!)
        del (self.source_path)

    def to_hdf(self, out_file=None, **args):
        """Save a stream out to a single hdf5 file.
        """
        if not out_file:
            out_file = str(int(self.start_datetime.timestamp())) + '.h5'

        # Default compression level
        compression_args = dict(complevel=9, complib='blosc:zstd')
        compression_args.update(args)

        with pd.HDFStore(out_file, **compression_args) as store:
            for name, stream in self.streams.items():
                key = '/dt/' + name
                store.put(key, stream.data)
                attrs = store.get_storer(key).attrs
                stream_metadata = dict(
                    start_timestamp=stream.start_timestamp,
                    frequency=stream.frequency,
                    filepath=stream.filepath,
                    duration=stream.duration)
                attrs.metadata = stream_metadata

            attrs = store.get_node('/dt')._v_attrs
            for key, val in self.e4_meta.items():
                attrs[key] = val

    def load_hdf(self, source_path):
        """Load a session from a saved h5 file"""
        streams = dict()
        with pd.HDFStore(source_path) as store:
            attrs = store.get_node('/dt/')._v_attrs
            self.e4_meta = attrs.__dict__

            for stream_name, (stream_class,
                              stream_path) in STREAM_SOURCE.items():
                stream_key = 'dt/' + stream_name

                # Load Dataset and H5 Stored Metadata
                stream_df = store.get(stream_key)
                stream_meta = store.get_storer(stream_key).attrs['metadata']

                # Create a thin stream (just the dataset)
                stream = stream_class(stream_df)

                # Then, update H5 Stored Metadata (FIXME - TOO FRAGILE)
                stream.start_timestamp = stream_meta['start_timestamp']
                stream.frequency = stream_meta['frequency']
                stream.filepath = stream_meta['filepath']
                stream.duration = stream_meta['duration']
                streams[stream_name] = stream

        self.streams = streams
        self._source_path, self._temp_source_path = None, None

    def to_archive(self, out_path=None):
        """Save a session to archive csv directory

        Parameters
        -----------
        out_path : str
            Path to use for saving the directory. Optionally end the file with
            ".zip" to create a zipped archive file, otherwise creates a
            directory structure.

        Returns
        -------
        out_path : str
            Returns the newly written archive path (in case default metadata
            was used to create the archive).
        """
        if not out_path:
            raise NotImplementedError(
                'need to set default dirname from metadata')
        if os.path.exists(out_path):
            raise IOError('Archive dir {} already exists - not overwriting'.
                          format(out_path))
        else:
            if out_path.endswith('.zip'):
                out_dir = out_path[:-4]
            else:
                out_dir = out_path
            os.makedirs(out_dir)

        for stream_name, (stream_class, stream_path) in STREAM_SOURCE.items():
            stream = self.streams[stream_name]
            temp_path = os.path.join(out_dir, stream_path)
            stream.to_csv(temp_path)

        if out_path.endswith('.zip'):
            with zipfile.ZipFile(out_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
                self._make_zip(out_dir, zipf)
            rmtree(out_dir)

        return out_path

    def _make_zip(self, path, ziph):
        path = os.path.abspath(path)
        # ziph is zipfile handle
        for root, dirs, files in os.walk(path):
            dest_dir = root.replace(os.path.dirname(path), '', 1)
            for file in files:
                ziph.write(
                    os.path.join(root, file),
                    arcname=os.path.join(dest_dir, file))

    def _load_meta(self, path):
        """Load vendor info from metadata.json if present.

        Be sure to call after header and data have been read so that they
        are available for consistency checks."""
        if os.path.exists(path):
            with open(path, 'r') as f:
                e4_info = json.load(f)

            duration = self.streams['acc'].duration
            if e4_info['duration'] != duration:
                err_pat = ('Duration from metadata ({}) does not '
                           'match duration from data ({})')
                err_args = e4_info['duration'], duration
                logger.warning(err_pat.format(*err_args))

            start_timestamp = self.streams['acc'].start_timestamp
            if e4_info['start_time'] != start_timestamp:
                err_pat = ('Starting timestamp from metadata ({}) '
                           'does not match start read from data ({})')
                err_args = e4_info['start_time'], start_timestamp
                logger.warning(err_pat.format(*err_args))

        else:
            e4_info = {}
        return e4_info

    # def load_from_db(self, physio_session):
    #     self._source_path, self._temp_source_path = None, None
    #     self.streams = dict()
    #
    #     stream_classes = [('acc', AccStream), ('bvp', BvpStream),
    #                       ('eda', EdaStream), ('hr', HrStream),
    #                       ('ibi', IbiStream), ('press', PressStream),
    #                       ('temperature', TemperatureStream)]
    #     for label, stream_cls in stream_classes:
    #         # # OPTIMIZE: Combine all columns into a single db call
    #         stream_profiles = models.StreamProfile.objects.filter(
    #             stream_name=label)
    #
    #         all_value_series = []
    #         for stream_profile in stream_profiles.all():
    #             stream_list = physio_session.streamlist_set.get(
    #                 stream_profile=stream_profile)
    #             records = stream_profile.value_class.objects.filter(
    #                 stream_list=stream_list)
    #             query = str(records.all().query)
    #             df = pd.read_sql_query(query, connection)
    #             df.index = df.record_time
    #             series = df['value']
    #             series.name = stream_profile.component_source_name
    #             all_value_series.append(series)
    #
    #         df = pd.DataFrame(all_value_series).T
    #         stream = stream_cls(df)
    #         self.streams[label] = stream
    #
    #     e4_meta = {}
    #     e4_meta['id'] = physio_session.empatica_id
    #     source = physio_session.source
    #     if source:
    #         e4_meta['device_id'] = source.device_serial
    #         e4_meta['label'] = source.device_label
    #         e4_meta['device'] = source.source_type_id.model_version
    #     self.e4_meta = e4_meta

    def hdf5like(self, source_path):
        try:
            pd.read_hdf(source_path)
            is_hdf = True
        except (NotImplementedError, OSError):
            is_hdf = False
        return is_hdf

    def default_fname(self, ext='.zip'):
        fname = "%d" % self.start_datetime.timestamp()
        if 'device_id' in self.e4_meta.keys():
            fname += '_' + self.e4_meta['device_id']
        fname += ext
        return fname


class Stream(object):
    """A single E4 stream (e.g. ACC, IBI, ...).

    General methods to handle oddities of the E4 data format and normalize
    across modalities.

    Parameters
    ---------
    filepath_or_buffer : file handle or path string
        Source E4 data, as either the path to a [possibly gzipped] file name
        or file handle (possibly existing only in memory).

    header_names : list
        Names of each columns measurements are required; see subclasses for
        more information.

    timestamp_present : bool, default True
        True if line 1 contains a timestamp for this stream format

    freq_present : bool, default True
        True if line 2 contains a frequency for this stream format


    Attributes
    ----------

    filepath : str or handle
        Source filepath used to generate the string.

    data : pd.DataFrame
        dataframe with time-stampped index and named columns.

    start_timestamp : int
        The initial timestamp from the csv (line 1) in UTC.

    frequency : int or None
        The initial frequency from the csv (ussually line 2, absent for IBI).

    duration : float
        Duration of stream recording (in seconds)
    """

    def __init__(self,
                 filepath_or_buffer,
                 header_names,
                 timestamp_present=True,
                 freq_present=True):
        if isinstance(filepath_or_buffer, pd.DataFrame):
            # If a dataframe is provided, set it directly w/o ts and frequency
            data = filepath_or_buffer

            if data.shape[0]:  # Not an empty Dataframe
                start_timestamp = int(data.index[0].timestamp())

                # Check to see if all time deltas are the same
                # ... (sampled with regular frequency)
                timesteps = (data.iloc[1:].index - data.iloc[:-1].index)
                if timesteps.unique().size == 1:
                    timestep = data.index[1] - data.index[0]
                    frequency = int(round(1 / timestep.total_seconds()))
                else:
                    frequency = None
            else:
                start_timestamp = None
                frequency = None

        else:
            if os.path.exists(filepath_or_buffer):
                self.filepath = filepath_or_buffer
            else:
                # If an in-memory buffer was passed don't set filepath.
                self.filepath = None

            data, start_timestamp, frequency = self.parse_raw_data(
                filepath_or_buffer, header_names, timestamp_present,
                freq_present)

        self.data = data
        self.start_timestamp = start_timestamp
        self.frequency = frequency
        self.timestamp_present = timestamp_present
        self.freq_present = freq_present

        # Calculate duration directly from Datetime index of `data`
        if self.data.shape[0]:
            self.duration = (data.index[-1] - data.index[0]).total_seconds()
        else:  # Except empty DataFrame
            self.duration = 0

    def parse_raw_data(self,
                       filepath_or_buffer,
                       header_names,
                       timestamp_present=True,
                       freq_present=True):
        """Read a single E4-formatted stream of data.

        Parameters
        ----------
        filepath_or_buffer : file handle or str
            Path or buffer to load stream data from

        header_names : list of strings
            Column names of a stream's specific attributes. Hard-coded
            when sub-classing Stream as in AccStream (['x', 'y', 'z']),
            TemperatureStream (['temperature']) etc.

        timestamp_present : boolean, default True
            Indicator for wehther or not line 1 is a timestamp (true except for
            phone press [tag] stream)

        freq_present : boolean, default True
            Indicator for whether or not line 2 is a frequency (true except for
            streams where the values themselves are timestamps, currently only
            IBI).


        Returns
        -------
        stream_df : pd.DataFrame
            Pandas dataframe with proper timestampped index and frequency.

        timestamp : int
            Unix timestamp (UTC) of stream start (row 1 when timestamp present).

        frequency : int
            Frequency in Hz of stream (row 2 when freq present).
        """
        timestamp, frequency, n_samples = self.read_header(
            filepath_or_buffer,
            timestamp_present=timestamp_present,
            freq_present=freq_present)

        if timestamp_present:
            if freq_present:
                skiprows = 2
            else:
                skiprows = 1
        else:
            skiprows = 0

        stream_df = pd.read_csv(
            filepath_or_buffer, skiprows=skiprows, names=header_names)

        # Almost always, there will be a frequency for the stream so
        # timestamps can be inferred. For the case of processed heartbeat data
        # (IBI) that's not the case so the index will be a standard
        # auto-incrementing int.
        if frequency:
            timestamp_index = self.build_freq_index(timestamp, frequency,
                                                    n_samples)
            if timestamp_index.size == 0:
                timestamp_msg = ("Couldn't build a valid "
                                 "timestamp index with (%s, %s, %s)")
                raise ValueError(
                    timestamp_msg % (timestamp, frequency, n_samples))
        else:
            if timestamp:
                timestamp_index = self.build_timedelta_index(
                    timestamp, stream_df)
            else:
                # Use only the first column (currently df is always 1-d anyway)
                timestamp_index = self.build_timestamp_index(
                    stream_df.iloc[:, 0])
        stream_df.index = timestamp_index
        return stream_df, timestamp, frequency

    def read_header(self,
                    filepath_or_buffer,
                    timestamp_present=True,
                    freq_present=True):
        """Go through the stream once to read some metadata

        This is the first of two passes simply to get metadata and sample
        count, so as an algorithm it could certainly be improved, but we don't
        need to optimize right now.

        Read E4-formatted header, where line 1 is the unix timestamp and line 2
        is ussually the frequency (when freq_present is True).

        nb E4 always appears to have a correct unix timestamp (a base unit of
        seconds), but the embrace file looks to have timestamps as a base unit
        of milliseconds.

        Parameters
        ----------
        filepath_or_buffer : file handle or str
            Path or buffer to load stream data from

        timestamp_present : boolean, default True
            Indicator for wehther or not line 1 is a timestamp (true except for
            phone press [tag] stream)

        freq_present : boolean, default True
            Indicator for whether or not line 2 is a frequency (true except for
            streams where the values themselves are timestamps, currenctly only
            IBI).

        Returns
        -------
        timestamp : int or None
            Initial unix timestamp (from line 1) in UTC or None if not present
            (press stream [tags.csv] )

        freq : int or None
            Frequency in Hz (from line 2) or None if not present (IBI).

        n_samples : int
            Count of remaining lines to build timestamp
        """
        if filepath_or_buffer.endswith('.gz'):
            openfunc = gzip.open
            delim = b','
        else:
            openfunc = open
            delim = ','

        with openfunc(filepath_or_buffer, 'r') as f:
            # Round-trip through float for pythonic reading of the number
            if timestamp_present:
                ts_val = f.readline().strip().split(delim)[0]
                try:
                    timestamp = int(float(ts_val))
                except ValueError:
                    timestamp = None
            else:
                timestamp = None

            if freq_present:
                ts_val = f.readline().strip().split(delim)[0]
                try:
                    freq = int(float(ts_val))
                except ValueError:
                    freq = None
            else:
                freq = None
            n_samples = 0
            while f.readline():
                n_samples += 1

        return timestamp, freq, n_samples

    def build_freq_index(self, timestamp, freq, n_samples):
        """Build a datetime index from initial timestamp, frequency, and number
        of samples."""
        start = datetime.datetime.fromtimestamp(timestamp, tz=pytz.utc)

        # Convert from int herz frequency to a pandas datetime offset string
        freq_string = "%fL" % (1 / freq * 1000)

        # Build the DatetimeIndex
        timestamp_index = pd.date_range(
            start=start, freq=freq_string, periods=n_samples)

        return timestamp_index

    def build_timedelta_index(self, timestamp, df):
        """Build a datetime index from an initial time and a df of time
        deltas."""

        # Insert 0 (to account for timestamp in the header)
        df.loc[0] = 0

        # Calculate the additive time (in seconds) from the df index
        sec_delta = df.index.sort_values().to_series().cumsum()

        # Create timestamps by adding the initial time
        timestamps = timestamp + sec_delta.values

        # Create a pd.Series where index is the delta and value is timestamp
        ts_series = pd.to_datetime(timestamps, unit='s', utc=True)

        return ts_series

    def build_timestamp_index(self, series):
        """Build a datetime index from a df of timestamps.

        At present can only handle a series (vector)"""
        ts_series = pd.to_datetime(series, unit='s', utc=True)
        return ts_series

    @property
    def num_samples(self):
        return self.data.shape[0]

    def to_csv(self, fname=None):
        frames = []
        if self.timestamp_present:
            ts_df = pd.DataFrame([], columns=self.data.columns)
            ts_df.loc[0] = self.start_timestamp
            frames.append(ts_df)

        if self.freq_present:
            freq_df = pd.DataFrame([], columns=self.data.columns)
            freq_df.loc[0] = self.frequency
            frames.append(freq_df)

        frames.append(self.data)

        out_df = pd.concat(frames)

        out_df.to_csv(fname, header=False, index=False)


class AccStream(Stream):
    def __init__(self, *args, **kwargs):
        kwargs['header_names'] = ['x', 'y', 'z']
        super(AccStream, self).__init__(*args, **kwargs)


class BvpStream(Stream):
    def __init__(self, *args, **kwargs):
        kwargs['header_names'] = ['bvp']
        super(BvpStream, self).__init__(*args, **kwargs)


class EdaStream(Stream):
    def __init__(self, *args, **kwargs):
        kwargs['header_names'] = ['eda']
        super(EdaStream, self).__init__(*args, **kwargs)


class HrStream(Stream):
    def __init__(self, *args, **kwargs):
        kwargs['header_names'] = ['hr']
        super(HrStream, self).__init__(*args, **kwargs)


class IbiStream(Stream):
    def __init__(self, *args, **kwargs):
        kwargs['header_names'] = ['ibi']
        kwargs['freq_present'] = False
        super(IbiStream, self).__init__(*args, **kwargs)


class PressStream(Stream):
    def __init__(self, *args, **kwargs):
        kwargs['header_names'] = ['press']
        kwargs['freq_present'] = False
        kwargs['timestamp_present'] = False
        super(PressStream, self).__init__(*args, **kwargs)


class TemperatureStream(Stream):
    def __init__(self, *args, **kwargs):
        kwargs['header_names'] = ['temperature']
        super(TemperatureStream, self).__init__(*args, **kwargs)


STREAM_SOURCE = dict(
    acc=(AccStream, 'ACC.csv'),
    bvp=(BvpStream, 'BVP.csv'),
    eda=(EdaStream, 'EDA.csv'),
    hr=(HrStream, 'HR.csv'),
    ibi=(IbiStream, 'IBI.csv'),
    press=(PressStream, 'tags.csv'),
    temperature=(TemperatureStream, 'TEMP.csv'),
)



from empatican import cli
from empatican.interface import connect
import io
import os
import shlex
import shutil
import sys
from unittest import TestCase

FIXTURES_DIR = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), 'fixtures')


class ConnectTest(TestCase):
    '''LIVE Interact with Empatica Connect'''

    def setUp(self):
        self.assignment_csv = os.path.join(FIXTURES_DIR, 'assignments.csv')

    def test_list_cli(self):
        '''Basic list / search'''
        args = shlex.split('list --n-recent 2')
        parser = cli.add_parser()
        options = parser.parse_args(args)

        saved_stdout = sys.stdout
        try:
            out = io.StringIO()
            sys.stdout = out
            connect.search_or_download(vars(options))
            output = out.getvalue().strip()
            self.assertIn('device', output)
        finally:
            sys.stdout = saved_stdout

    def test_list_withassignment_cli(self):
        '''Search using device assignment table'''

        # With filenames, shlex isn't good with filenames including spaces
        # so skip it and explicilty use a list
        args = [
            'list', '--device-assignment', self.assignment_csv,
            '--participant-id', 'xyz003'
        ]
        print(args)
        parser = cli.add_parser()
        options = parser.parse_args(args)

        saved_stdout = sys.stdout
        try:
            out = io.StringIO()
            sys.stdout = out
            connect.search_or_download(vars(options))
            output = out.getvalue().strip()
            self.assertIn('device', output)
        finally:
            sys.stdout = saved_stdout

    def test_download_cli(self):
        '''Download small session with default filename'''
        # Download a 1 minute session
        args = shlex.split('download --empatica-id 565082')
        parser = cli.add_parser()
        options = parser.parse_args(args)

        zipfile = '1539959313_849318.zip'

        saved_stdout = sys.stdout
        try:
            out = io.StringIO()
            sys.stdout = out
            connect.search_or_download(vars(options))
            output = out.getvalue().strip()
            self.assertIn('Success', output)
            self.assertTrue(os.path.exists(zipfile))

        finally:
            sys.stdout = saved_stdout
            if os.path.exists(zipfile):
                os.remove(zipfile)

    def test_download_withassignment_cli(self):
        '''Download small session with study template'''

        template = 'study1/{participant_id}/{start_time}_{device_id}.zip'
        participant_id = 'xyz004'
        zipfile = template.format(
            participant_id=participant_id,
            start_time='1539959313',
            device_id='849318')
        # Download a 1 minute session
        args = [
            'download', '--empatica-id', '565082', '--device-assignment',
            self.assignment_csv, '--participant-id', participant_id,
            '--template', template
        ]
        parser = cli.add_parser()
        options = parser.parse_args(args)

        saved_stdout = sys.stdout
        try:
            out = io.StringIO()
            sys.stdout = out
            connect.search_or_download(vars(options))
            output = out.getvalue().strip()
            self.assertIn('Success', output)
            self.assertTrue(os.path.exists(zipfile))
        finally:
            sys.stdout = saved_stdout
            if os.path.exists('study1'):
                shutil.rmtree('study1')
            if os.path.exists(zipfile):
                os.remove(zipfile)



#!/usr/bin/env python
# Simple E4 Downloader
from getpass import getpass
import json
import os
import pandas as pd
import re
import requests
import sys
import zipfile

BASE_URL = 'https://www.empatica.com/connect/'
AUTH_URL = BASE_URL + 'authenticate.php'
DOWNLOAD_URL = BASE_URL + 'download.php'
SESSIONS_URL = BASE_URL + 'connect.php/users/{}/sessions'

if 'EMPATICA_USER' in os.environ.keys():
    USERNAME = os.environ['EMPATICA_USER']
    PASSWORD = os.environ['EMPATICA_PASS']
else:
    USERNAME = input('Empatica Username: ')
    PASSWORD = getpass('Empatica Password: ')


def search_or_download(options):
    '''Simple wrapper to open and use a request session

    Parameters
    -----------
        options : dict
            Dictionary with options from :class:`empatican.cli` list or
            download parser (e.g. participant_id, device_id,
            device_assignment). See the :ref:`commandline-reference` for more
            information.
    '''
    if options['participant_id'] and not options['device_assignment']:
        msg = ('Error: A --device-assignment csv table is required if '
               'specifying --participant-id')
        raise ValueError(msg)

    with requests.Session() as req_session:
        _search_or_download(req_session, options)


def _search_or_download(req_session, options):
    '''Main handler for searching and downloading E4 sessions'''
    # Logs in to the E4 connect site and updates req_session in place
    login(req_session)

    # Get a list of all sessions by querying the econnect site (or cache)
    session_list = get_sessions(req_session, json_cache=None, **options)
    session_df = pd.DataFrame(session_list)

    if options['device_assignment']:
        # If device assignment was provided, add a participant_id column inline
        merge_device_assignment(session_df, options['device_assignment'])

    filtered_df = filter_sessions(session_df, options)

    if options['action'] == 'list':
        if options['table_fmt'] == 'tab':
            table_string = filtered_df.to_string() + '\n'
        elif options['table_fmt'] == 'csv':
            table_string = filtered_df.to_csv()
        else:
            raise ValueError('Format must be either "tab" or "csv"')

        sys.stdout.write(table_string)

    elif options['action'] == 'download':
        for _, row in filtered_df.iterrows():
            try:
                if options['template']:
                    local_filename = build_fname(row, options)
                else:  # Use default within download_session
                    local_filename = None

                download_fname = download_session(
                    row, req_session, local_filename=local_filename)
                sys.stdout.write(
                    'Successfully downloaded "%s"\n' % download_fname)
            except Exception as e:
                sys.stdout.write(
                    'Problem downloading "%s": %s\n' % (row['id'], e))


def filter_sessions(session_df, options):
    # Create a set of filters to restrict sessions
    query = []
    if options['device_id']:
        device_id = options['device_id']
        query.append("device_id == @device_id")
    if options['empatica_id']:
        empatica_id = options['empatica_id']
        query.append("id == @empatica_id")
    if options['participant_id']:
        participant_id = options['participant_id']
        query.append("participant_id == @participant_id")
    if options['start_date']:
        start_date = options['start_date']
        query.append("start_datetime >= @start_date")
    if options['end_date']:
        end_date = options['end_date']
        query.append("start_datetime <= @end_date")

    if options['n_recent']:
        n_recent = options['n_recent']
    else:
        n_recent = session_df.shape[0]  # (don't tail any)

    # Take the filters and apply them once
    if len(query):
        session_df.query(' & '.join(query), inplace=True)

    # Filter n recent sessions in place
    session_df = session_df.iloc[-n_recent:]
    return session_df


def merge_device_assignment(session_df, device_assignment_sheet):
    """Add participant_id from device assignment table

    Expects a sheet with columns *participant_id*, *start_date*, *end_date* and
    *device_serial*

    Dates must be fully-valid dates (e.g. including year), Day-Month aren't
    sufficient."""
    csv_df = pd.read_csv(
        device_assignment_sheet, parse_dates=['start_date', 'end_date'])

    # Initialize 'participant_id' column
    session_df.loc[:, 'participant_id'] = None

    # Add participant_id in session from matching assignment rows.
    for row_idx, row in session_df.iterrows():
        search = ((csv_df['device_serial'] == row['device_name']) &
                  (csv_df['start_date'] <= row['start_datetime'])
                  & (csv_df['end_date'] >= row['start_datetime']))
        device_assignment_rows = csv_df.loc[search]
        if device_assignment_rows.shape[0] > 0:
            cur_participant_id = '_or_'.join(
                device_assignment_rows['participant_id'].astype(str).values)
            session_df.loc[row_idx, 'participant_id'] = cur_participant_id
        else:
            continue


def build_fname(row, options):
    '''Build an output file from a format.'''
    # TODO Add sanity check format, handle errors
    if not options['template'].endswith('.zip'):
        options['template'] += '.zip'

    try:
        fname = options['template'].format(**row)
    except KeyError as e:
        print(e)
        raise FormatError(
            "Problem with your template string. "
            "Be sure to use only the following special values: {}".format(
                row.keys()))
    return fname


def login(req_session, login_info=None):
    if not login_info:
        login_info = dict(username=USERNAME, password=PASSWORD)

    # Login with the session
    response = req_session.post(AUTH_URL, data=login_info)
    if 'dashboard' not in response.url:
        msg = "Couldn't log in {}. Check username/pass and try again.".format(
            login_info['username'])
        raise ValueError(msg)


def get_userid(req_session):
    recent_sessions = req_session.get(BASE_URL + 'sessions.php')
    match = re.search('var userId = (\d+)', recent_sessions.text)
    userid = match.groups()[0]
    return userid


def get_sessions(req_session, json_cache='sessions_list.json', **options):
    '''Download or load from cache a list of session dictionaries.
    To avoid any caching, use json_cache=None'''
    # Determine the userid if not provided explicitly
    # (one fewer REST call if pre-determined)
    if 'userid' not in options.keys() or not options['userid']:
        userid = get_userid(req_session)
        options['userid'] = userid
        # print('Found user as {}'.format(userid))

    # Build the sessions list url and grab the full list of sessions
    sessions_list_url = SESSIONS_URL.format(options['userid'])

    if json_cache:
        if os.path.exists(json_cache):
            with open(json_cache, 'r') as f:
                sessions_list = json.load(f)
        else:
            sessions_list_response = req_session.get(sessions_list_url)
            sessions_list = sessions_list_response.json()
            with open(json_cache, 'w') as f:
                indent = None
                json.dump(sessions_list, f, indent=indent)

    else:
        sessions_list_response = req_session.get(sessions_list_url)
        sessions_list = sessions_list_response.json()

    session_df = pd.DataFrame(sessions_list)

    session_df['start_datetime'] = pd.to_datetime(
        session_df['start_time'], unit='s')
    session_df['_duration'] = session_df['duration'].astype(int).apply(
        _pretty_relative_time)

    # Empatica stores hex device names as ints in their site. For all E4
    # devices, convert the "label" into a 5-padded hex string plus "A" prefix
    session_df['device_name'] = session_df['label'].dropna().astype(int).apply(
        "A{0:05x}".format).str.upper()

    session_df.drop(
        columns=['status', 'exit_code'], inplace=True, errors='ignore')

    return session_df


def download_session(physio_session, requests_session, local_filename=None):
    """Download a single session from Empatica Connect

    Parameters
    -----------
    physio_session : Dictlike, pandas row is acceptable
        Session details with metadata keys from empatica connect site. **id**
        is the only required key, but other information may be provided as
        well to automatically create a `empatica connect`-style zip filename,
        e.g. keys 'start_time' and 'device_id'.

    requests_session : :class:requests.Session
        An open and logged-in ``Session``

    local_filename : str, optional
        An optional filename may be provided. If dirname does not exist, it
        will be created recursively. Leave as ``None`` to infer a standard
        filename from ``options`` as ``{start_time}_{device_id}.zip``.

    Returns
    --------

    local_filename : str, default None
        The filename of the zipfile created, regardless of whether the filename
        was inferred or passed in explicitly.

    """
    if not local_filename:
        local_filename = '{start_time}_{device_id}.zip'.format(
            **physio_session)

    # Recursively create output directory if needed
    local_dirname = os.path.abspath(os.path.dirname(local_filename))
    if not os.path.exists(local_dirname):
        os.makedirs(local_dirname)

    params = dict(id=physio_session['id'])

    if not os.path.exists(local_filename):
        with requests_session.get(
                DOWNLOAD_URL, params=params, stream=True) as stream:
            with open(local_filename, 'wb') as f:
                for chunk in stream.iter_content(chunk_size=512):
                    # if chunk:  # filter out keep-alive new chunks
                    f.write(chunk)
            write_metadata(physio_session, local_filename)
    return local_filename


def write_metadata(physio_session, local_filename):
    """Append metadata from download to zip"""
    if isinstance(physio_session, pd.Series):
        meta = physio_session.to_dict()
    else:
        meta = physio_session
    with zipfile.ZipFile(local_filename, 'a') as z:
        z.writestr(
            'metadata.json',
            json.dumps(meta, indent=4, sort_keys=True, default=str),
        )


def download(req_session, **options):
    payload = dict(username=USERNAME, password=PASSWORD)

    # Login with the session
    req_session.post(AUTH_URL, data=payload)

    sessions_list = get_sessions(req_session, options)

    if 'n_recent' in options.keys():
        n_recent = sessions_list.shape[0]

    for _, physio_info in sessions_list.tail(n_recent).iterrows():
        download_session(physio_info, req_session)


def _pretty_relative_time(time_diff_secs):
    '''Originally from: https://stackoverflow.com/questions/1551382/user-friendly-time-format-in-python
    Each tuple in the sequence gives the name of a unit, and the number of
    previous units which go into it.
    '''
    weeks_per_month = 365.242 / 12 / 7
    intervals = [('minute', 60), ('hour', 60), ('day', 24), ('week', 7),
                 ('month', weeks_per_month), ('year', 12)]

    unit, number = 'second', abs(time_diff_secs)
    for new_unit, ratio in intervals:
        new_number = float(number) / ratio
        # If the new number is too small, don't go to the next unit.
        if new_number < 2:
            break
        unit, number = new_unit, new_number
    shown_num = int(number)
    return '{} {}'.format(shown_num, unit + ('' if shown_num == 1 else 's'))


class FormatError(Exception):
    pass



from setuptools import setup

setup(
    name='empatican',
    version='0.0.2',
    description='Empatica Physio Utilities',
    url='http://github.com/kastman/empatican',
    author='Erik Kastman',
    author_email='erik.kastman@gmail.com',
    packages=['empatican', 'empatican.interface', 'empatican.physio'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Utilities',
    ],
    entry_points={
        'console_scripts': ['empatican=empatican.cli:main'],
    },
    license='MIT',
    install_requires=['requests', 'pandas'],
    test_suite='nose.collector',
    tests_require=['nose'],
    zip_safe=True)
